@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,7 +26,9 @@ require_once dirname(__FILE__).'/include/blocks.inc.php';
 
 $page['title'] = _('Configuration of screens');
 $page['file'] = 'screenedit.php';
-$page['scripts'] = ['class.cscreen.js', 'class.calendar.js', 'gtlc.js', 'flickerfreescreen.js', 'multiselect.js'];
+$page['scripts'] = ['class.cscreen.js', 'class.calendar.js', 'gtlc.js', 'class.svg.canvas.js', 'class.svg.map.js',
+	'flickerfreescreen.js', 'multiselect.js'
+];
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
 
 require_once dirname(__FILE__).'/include/page_header.php';
@@ -36,12 +38,12 @@ $knownResourceTypes = [
 	SCREEN_RESOURCE_SIMPLE_GRAPH,
 	SCREEN_RESOURCE_MAP,
 	SCREEN_RESOURCE_PLAIN_TEXT,
-	SCREEN_RESOURCE_HOSTS_INFO,
-	SCREEN_RESOURCE_TRIGGERS_INFO,
+	SCREEN_RESOURCE_HOST_INFO,
+	SCREEN_RESOURCE_TRIGGER_INFO,
 	SCREEN_RESOURCE_SERVER_INFO,
 	SCREEN_RESOURCE_CLOCK,
 	SCREEN_RESOURCE_SCREEN,
-	SCREEN_RESOURCE_TRIGGERS_OVERVIEW,
+	SCREEN_RESOURCE_TRIGGER_OVERVIEW,
 	SCREEN_RESOURCE_DATA_OVERVIEW,
 	SCREEN_RESOURCE_URL,
 	SCREEN_RESOURCE_ACTIONS,
@@ -268,7 +270,7 @@ $data = [
 	'screenid' => getRequest('screenid', 0)
 ];
 
-// getting updated screen, so we wont have to refresh the page to see changes
+// Getting updated screen, so we won't have to refresh the page to see changes.
 $data['screen'] = API::Screen()->get($options);
 if (empty($data['screen'])) {
 	$data['screen'] = API::TemplateScreen()->get($options);
