@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -79,7 +79,7 @@ const char	*help_message[] = {
 	"Utility for sending monitoring data to Zabbix server or proxy.",
 	"",
 	"General options:",
-	"  -c --config config-file    Absolute path to Zabbix agentd configuration file",
+	"  -c --config config-file    Path to Zabbix agentd configuration file",
 	"",
 	"  -z --zabbix-server server  Hostname or IP address of Zabbix server or proxy",
 	"                             to send data to. When used together with --config,",
@@ -239,21 +239,38 @@ static char	*INPUT_FILE = NULL;
 static int	WITH_TIMESTAMPS = 0;
 static int	REAL_TIME = 0;
 
-static char	*CONFIG_SOURCE_IP = NULL;
-static char	*ZABBIX_SERVER = NULL;
-unsigned short	ZABBIX_SERVER_PORT = 0;
-static char	*ZABBIX_HOSTNAME = NULL;
-static char	*ZABBIX_KEY = NULL;
-static char	*ZABBIX_KEY_VALUE = NULL;
+static char		*CONFIG_SOURCE_IP = NULL;
+static char		*ZABBIX_SERVER = NULL;
+static unsigned short	ZABBIX_SERVER_PORT = 0;
+static char		*ZABBIX_HOSTNAME = NULL;
+static char		*ZABBIX_KEY = NULL;
+static char		*ZABBIX_KEY_VALUE = NULL;
 
 #if !defined(_WINDOWS)
 static void	send_signal_handler(int sig)
 {
-	if (SIGALRM == sig)
-		zabbix_log(LOG_LEVEL_WARNING, "timeout while executing operation");
+
+#define CASE_LOG_WARNING(signal) \
+	case signal:							\
+		zabbix_log(LOG_LEVEL_WARNING, "interrupted by signal " #signal " while executing operation"); \
+		break
+
+	switch (sig)
+	{
+		CASE_LOG_WARNING(SIGALRM);
+		CASE_LOG_WARNING(SIGINT);
+		CASE_LOG_WARNING(SIGQUIT);
+		CASE_LOG_WARNING(SIGTERM);
+		CASE_LOG_WARNING(SIGHUP);
+		CASE_LOG_WARNING(SIGPIPE);
+		default:
+			zabbix_log(LOG_LEVEL_WARNING, "signal %d while executing operation", sig);
+	}
+#undef CASE_LOG_WARNING
 
 	/* Calling _exit() to terminate the process immediately is important. See ZBX-5732 for details. */
-	_exit(EXIT_FAILURE);
+	/* Return FAIL instead of EXIT_FAILURE to keep return signals consistent for send_value() */
+	_exit(FAIL);
 }
 #endif
 
@@ -284,7 +301,8 @@ ZBX_THREAD_SENDVAL_ARGS;
  ******************************************************************************/
 static int	update_exit_status(int old_status, int new_status)
 {
-	if (FAIL == old_status || FAIL == new_status || (unsigned char)FAIL == new_status)
+	/* FAIL(-1) signal gets converted to unsigned char when leaving the thread */
+	if (FAIL == old_status || (unsigned char)FAIL == new_status)
 		return FAIL;
 
 	if (SUCCEED == old_status)
@@ -425,12 +443,12 @@ static int	check_response(char *response)
 	ret = zbx_json_open(response, &jp);
 
 	if (SUCCEED == ret)
-		ret = zbx_json_value_by_name(&jp, ZBX_PROTO_TAG_RESPONSE, value, sizeof(value));
+		ret = zbx_json_value_by_name(&jp, ZBX_PROTO_TAG_RESPONSE, value, sizeof(value), NULL);
 
 	if (SUCCEED == ret && 0 != strcmp(value, ZBX_PROTO_VALUE_SUCCESS))
 		ret = FAIL;
 
-	if (SUCCEED == ret && SUCCEED == zbx_json_value_by_name(&jp, ZBX_PROTO_TAG_INFO, info, sizeof(info)))
+	if (SUCCEED == ret && SUCCEED == zbx_json_value_by_name(&jp, ZBX_PROTO_TAG_INFO, info, sizeof(info), NULL))
 	{
 		int	failed;
 
@@ -465,10 +483,12 @@ static	ZBX_THREAD_ENTRY(send_value, args)
 #endif
 
 #if !defined(_WINDOWS)
-	signal(SIGINT,  send_signal_handler);
-	signal(SIGTERM, send_signal_handler);
+	signal(SIGINT, send_signal_handler);
 	signal(SIGQUIT, send_signal_handler);
+	signal(SIGTERM, send_signal_handler);
+	signal(SIGHUP, send_signal_handler);
 	signal(SIGALRM, send_signal_handler);
+	signal(SIGPIPE, send_signal_handler);
 #endif
 	switch (configured_tls_connect_mode)
 	{
@@ -982,7 +1002,7 @@ static char	*zbx_fgets_alloc(char **buffer, size_t *buffer_alloc, FILE *fp)
 		if (*buffer_alloc - buffer_offset < len + 1)
 		{
 			*buffer_alloc = (buffer_offset + len + 1) * 3 / 2;
-			*buffer = zbx_realloc(*buffer, *buffer_alloc);
+			*buffer = (char *)zbx_realloc(*buffer, *buffer_alloc);
 		}
 
 		memcpy(*buffer + buffer_offset, tmp, len);
@@ -1002,7 +1022,7 @@ int	main(int argc, char **argv)
 {
 	FILE			*in;
 	char			*in_line = NULL, hostname[MAX_STRING_LEN], key[MAX_STRING_LEN], *key_value = NULL,
-				clock[32];
+				clock[32], *error = NULL;
 	int			total_count = 0, succeed_count = 0, buffer_count = 0, read_more = 0, ret = FAIL,
 				timestamp;
 	size_t			in_line_alloc = MAX_BUFFER_LEN, key_value_alloc = 0;
@@ -1018,8 +1038,20 @@ int	main(int argc, char **argv)
 
 	zbx_load_config(CONFIG_FILE);
 
-	zabbix_open_log(LOG_TYPE_UNDEFINED, CONFIG_LOG_LEVEL, NULL);
-
+	if (SUCCEED != zabbix_open_log(LOG_TYPE_UNDEFINED, CONFIG_LOG_LEVEL, NULL, &error))
+	{
+		zbx_error("cannot open log: %s", error);
+		zbx_free(error);
+		exit(EXIT_FAILURE);
+	}
+#if defined(_WINDOWS)
+	if (SUCCEED != zbx_socket_start(&error))
+	{
+		zbx_error(error);
+		zbx_free(error);
+		exit(EXIT_FAILURE);
+	}
+#endif
 #if !defined(_WINDOWS) && (defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL))
 	if (SUCCEED != zbx_coredump_disable())
 	{
@@ -1100,7 +1132,7 @@ int	main(int argc, char **argv)
 		}
 
 		sendval_args.sync_timestamp = WITH_TIMESTAMPS;
-		in_line = zbx_malloc(NULL, in_line_alloc);
+		in_line = (char *)zbx_malloc(NULL, in_line_alloc);
 
 		ret = SUCCEED;
 
@@ -1163,7 +1195,7 @@ int	main(int argc, char **argv)
 			if (key_value_alloc != in_line_alloc)
 			{
 				key_value_alloc = in_line_alloc;
-				key_value = zbx_realloc(key_value, key_value_alloc);
+				key_value = (char *)zbx_realloc(key_value, key_value_alloc);
 			}
 
 			if ('\0' != *p && '"' != *p)
@@ -1310,7 +1342,10 @@ exit:
 	}
 #endif
 	zabbix_close_log();
-
+#if defined(_WINDOWS)
+	while (0 == WSACleanup())
+		;
+#endif
 	if (FAIL == ret)
 		ret = EXIT_FAILURE;
 
